==> bayonne2-0.6.2.tar.gz: bayonne2-0.6.2/server/btsexec.c <==
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include "private.h"

static char btspaths[] = CONFIG_FILES "/server.conf";
static char varpath[128] = VAR_FILES;
static char audio[128] = PROMPT_FILES;
static char libexec[128] = LIBEXEC_FILES;
static char control[128] = RUN_FILES;

static unsigned strlower(char *cp)
{
	unsigned len = 0;
	while(isalnum(*cp))
	{
		*cp = tolower(*cp);
		++cp;
		++len;
	}
	return len;
}

static char *getpath(char *cp)
{
	char *ep;
	if(isspace(*cp) || *cp == '=')
		++cp;
	ep = strchr(cp, ';');
	if(!ep)
		ep = strchr(cp, '#');
	if(!ep)
		ep = cp + strlen(cp) - 1;
	else
	{
		*ep = 0;
		--ep;
	}
	while(ep > cp && isspace(*ep))
	{
		*ep = 0;
		--ep;
	}
	return cp;
}
	
int main(int argc, char **argv)
{
	char lbuf[256];
	char *cp;
	unsigned len;
	char *ext = NULL;
	char *program = "/bin/sh";

	FILE *fp = fopen(btspaths, "r");
	if(!fp)
		fp = fopen("/etc/bayonne/server.conf", "r");
	if(!fp)
		goto launch;

	for(;;)
	{
		fgets(lbuf, sizeof(lbuf), fp);
		if(feof(fp))
		{
			fclose(fp);
			goto launch;
		}
		cp = lbuf;		
		while(isspace(*cp))
			++cp;

		if(*cp != '[')
			continue;

		++cp;
		while(isspace(*cp))
			++cp;
		len = strlower(cp);
		if(!len)
			continue;
		if(!strncmp(cp, "paths", len))
			break;
	}
	for(;;)
	{
		fgets(lbuf, sizeof(lbuf), fp);
		if(feof(fp))
			break;

		cp = lbuf;
		while(isspace(*cp))
			++cp;
		if(*cp == '[')
			break;
		if(!isalpha(*cp))
			continue;
		len = strlower(cp);
		if(!strncmp(cp, "libexec", len))
			strcpy(libexec, getpath(cp + len));
		else if(!strncmp(cp, "datafiles", len))
			strcpy(varpath, getpath(cp + len));
		else if(!strncmp(cp, "prompts", len))
			strcpy(audio, getpath(cp + len));
		else if(!strncmp(cp, "runfiles", len))
			strcpy(control, getpath(cp + len));
	}
	fclose(fp);		
	
launch:
	setenv("BAYONNE_HOME", varpath, 1);
	setenv("BAYONNE_AUDIO", audio, 1);
	setenv("BAYONNE_LIBEXEC", libexec, 1);
	setenv("BAYONNE_CONFIG", btspaths, 1);
	setenv("BAYONNE_CONTROL", control, 1);
	setenv("PERL5LIB", libexec, 1);
	setenv("PYTHONLIB", libexec, 1);

	if(argv[1])
		ext = strrchr(argv[1], '.');
	if(ext)
	{
		if(!strcmp(ext, ".pl"))
			program = PERL_PROGRAM;
		else if(!strcmp(ext, ".py"))
			program = PYTHON_PROGRAM;
		else if(!strcmp(ext, ".php"))
			program = PHP_PROGRAM;
	}
	ext = strrchr(argv[0], '-');
	if(ext)
	{
		if(!strcmp(ext, "-perl"))
			program = PERL_PROGRAM;
		else if(!strcmp(ext, "-python"))
			program = PYTHON_PROGRAM;
		else if(!strcmp(ext, "-php"))
			program = PHP_PROGRAM;
	}
	argv[0] = program;
	execv(program, argv);
	exit(-1);
}

==> bayonne2-0.6.3.tar.gz: bayonne2-0.6.3/server/btsexec.c <==
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <grp.h>
#include "private.h"

static char btspaths[] = CONFIG_FILES "/server.conf";
static char varpath[128] = VAR_FILES;
static char ramfs[128] = "/dev/shm/.bayonne";
static char tmpfs[128] = "/tmp/.bayonne";
static char audio[128] = PROMPT_FILES;
static char libexec[128] = LIBEXEC_FILES;
static char control[128] = RUN_FILES;

static unsigned strlower(char *cp)
{
	unsigned len = 0;
	while(isalnum(*cp))
	{
		*cp = tolower(*cp);
		++cp;
		++len;
	}
	return len;
}

static char *getpath(char *cp)
{
	char *ep;
	if(isspace(*cp) || *cp == '=')
		++cp;
	ep = strchr(cp, ';');
	if(!ep)
		ep = strchr(cp, '#');
	if(!ep)
		ep = cp + strlen(cp) - 1;
	else
	{
		*ep = 0;
		--ep;
	}
	while(ep > cp && isspace(*ep))
	{
		*ep = 0;
		--ep;
	}
	return cp;
}
	
int main(int argc, char **argv)
{
	char lbuf[256];
	char *cp;
	unsigned len;
	char *ext = NULL;
	char *program = "/bin/sh";
	struct group *grp;

	if(access("/dev/shm", W_OK))
		strcpy(ramfs, tmpfs);

	umask(007);
	grp = getgrnam("bayonne");
	if(grp)
		setgid(grp->gr_gid);
	endgrent();

	FILE *fp = fopen(btspaths, "r");
	if(!fp)
		fp = fopen("/etc/bayonne/server.conf", "r");
	if(!fp)
		goto launch;

	for(;;)
	{
		fgets(lbuf, sizeof(lbuf), fp);
		if(feof(fp))
		{
			fclose(fp);
			goto launch;
		}
		cp = lbuf;		
		while(isspace(*cp))
			++cp;

		if(*cp != '[')
			continue;

		++cp;
		while(isspace(*cp))
			++cp;
		len = strlower(cp);
		if(!len)
			continue;
		if(!strncmp(cp, "paths", len))
			break;
	}
	for(;;)
	{
		fgets(lbuf, sizeof(lbuf), fp);
		if(feof(fp))
			break;

		cp = lbuf;
		while(isspace(*cp))
			++cp;
		if(*cp == '[')
			break;
		if(!isalpha(*cp))
			continue;
		len = strlower(cp);
		if(!strncmp(cp, "libexec", len))
			strcpy(libexec, getpath(cp + len));
		else if(!strncmp(cp, "datafiles", len))
			strcpy(varpath, getpath(cp + len));
		else if(!strncmp(cp, "prompts", len))
			strcpy(audio, getpath(cp + len));
		else if(!strncmp(cp, "runfiles", len))
			strcpy(control, getpath(cp + len));
		else if(!strncmp(cp, "tmpfs", len))
			strcpy(ramfs, getpath(cp + len));
		else if(!strncmp(cp, "tmp", len))
			strcpy(tmpfs, getpath(cp + len));
	}
	fclose(fp);		
	
launch:
	setenv("BAYONNE_HOME", varpath, 1);
	setenv("BAYONNE_AUDIO", audio, 1);
	setenv("BAYONNE_LIBEXEC", libexec, 1);
	setenv("BAYONNE_CONFIG", btspaths, 1);
	setenv("BAYONNE_CONTROL", control, 1);
	setenv("BAYONNE_TMPFS", tmpfs, 1);
	setenv("BAYONNE_RAMFS", ramfs, 1);
	setenv("PERL5LIB", libexec, 1);
	setenv("PYTHONLIB", libexec, 1);

	if(argv[1])
		ext = strrchr(argv[1], '.');
	if(ext)
	{
		if(!strcmp(ext, ".pl"))
			program = PERL_PROGRAM;
		else if(!strcmp(ext, ".py"))
			program = PYTHON_PROGRAM;
		else if(!strcmp(ext, ".php"))
			program = PHP_PROGRAM;
	}
	ext = strrchr(argv[0], '-');
	if(ext)
	{
		if(!strcmp(ext, "-perl"))
			program = PERL_PROGRAM;
		else if(!strcmp(ext, "-python"))
			program = PYTHON_PROGRAM;
		else if(!strcmp(ext, "-php"))
			program = PHP_PROGRAM;
	}
	argv[0] = program;
	execv(program, argv);
	exit(-1);
}
